/*
 * timemgt.c
 *
 *  Created on: Apr 28, 2021
 *      Author: Florian
 */

#include "timemgt.h"

/* Local static variables */
tTimeMgt_Element Channels_tA[TIM_ChannelNumberMax_U8];
static tTimeMgt_List list_T;

static uint8_t channels_cnt_u8;
static uint32_t id_cnt_u32; // Free running counter - never decremented

/* Local static functions */
BOOL CompareStrings (char *string1, char *string2)
{
	BOOL ret_b = FALSE;

	return ret_b;
}

/* Functions */
void TimeMgt_Init (void)
{
	list_T.head = NULL;

	channels_cnt_u8 = 0U;
	id_cnt_u32 = 0U;
}

void TimeMgt_Start (void)
{

}

void TimeMgt_Stop(void)
{
	uint8_t cnt = 0U;

	for (cnt=0U; cnt < TIM_ChannelNumberMax_U8; cnt++)
	{
		TimeMgt_DisableCyclicEvent(cnt);
		TimeMgt_RemoveCyclicEvent(cnt);
	}
}

BOOL TimeMgt_AddCyclicEvent(uint8_t channel_u8, pTimeMgtCallback callback_p, uint32_t timeout_u32, char *tag_u8p, uint32_t *id_u32p)
{
	BOOL ret_b = FALSE;

	// Chained list managed
	if (channels_cnt_u8 < cTimeMgtChannelMax)
	{
		tTimeMgt_Element *element = malloc(sizeof(tTimeMgt_Element));
		tTimeMgt_Channel *channel = malloc(sizeof(tTimeMgt_Channel));

		element->item_p = channel;

		if (list_T.head == NULL) // If no object present in the list
		{
			element->next_p = NULL;
			list_T.head = element;
		}
		else // If objects present in the list
		{
			element->next_p = list_T.head;
			list_T.head = element;
		}

		((tTimeMgt_Channel*)(element->item_p))->enable_b = TRUE;
		((tTimeMgt_Channel*)(element->item_p))->counter_u32 = 0U;
		((tTimeMgt_Channel*)(element->item_p))->timeout_u32 = timeout_u32;
		((tTimeMgt_Channel*)(element->item_p))->callback_p = callback_p;

		// Increment channel counter
		channels_cnt_u8++;

		// Assign unique ID
		*id_u32p = id_cnt_u32;
		element->id_u32 = id_cnt_u32;
		id_cnt_u32++;

		// Assign tag
		uint8_t cnt_u8 = 0;
		BOOL end_of_line = FALSE;

		for (cnt_u8 = 0; cnt_u8 < cTimeMgtTagStringMaxLength; cnt_u8++)
		{
			if (*(tag_u8p + cnt_u8) == (char)0x00)
			{
				end_of_line = TRUE;
			}

			if (end_of_line == FALSE)
			{
				element->tag_u8a[cnt_u8] = *(tag_u8p + cnt_u8);
			}
			else
			{
				element->tag_u8a[cnt_u8] = (char)0x00;
			}

		}

		ret_b = TRUE;
	}

	return (ret_b);
}

BOOL TimeMgt_SearchEventById(uint32_t id_u32, tTimeMgt_Element** prev, tTimeMgt_Element** match)
{
	BOOL ret_b = FALSE;

	if(list_T.head != NULL) // If list is not empty
	{
		tTimeMgt_Element *element = list_T.head;

		if (element->id_u32 == id_u32) // Specific test for head
		{
			*match = element;
			*prev = NULL;
			ret_b = TRUE;
		}
		else  // If more than one element in list
		{
			do
			{
				if (element->next_p != NULL)
				{
					if (element->next_p->id_u32 == id_u32)
					{
						*match = (element->next_p);
						*prev = element;
						ret_b = TRUE;
						break;
					}
				}

				element = element->next_p;
			}
			while (element != NULL);
		}
	}
	else
	{
		*prev = NULL;
		*match = NULL;
	}

	return (ret_b);
}

void TimeMgt_RemoveCyclicEvent(uint8_t channel_u8)
{
	// Chained list managed
	if(list_T.head != NULL)
	{
		tTimeMgt_Element *element = list_T.head;
		list_T.head = list_T.head->next_p;
		free(element);
	}
}

BOOL TimeMgt_RemoveCyclicEventById(uint32_t id_u32)
{
	BOOL ret_b = FALSE;

	tTimeMgt_Element *match;
	tTimeMgt_Element *prev;

	if (TimeMgt_SearchEventById(id_u32, &prev, &match) == TRUE) // Search for the object
	{
		if (prev == NULL) // Match is HEAD
		{
			tTimeMgt_Element *element = list_T.head;
			list_T.head = list_T.head->next_p;
			free(element);
			ret_b = TRUE;
		}
		else if (match != NULL)
		{
			prev->next_p = match->next_p; // Bypass object to be removed
			free(match); // Free object that must be removed
			ret_b = TRUE;
		}
	}

	return (ret_b);
}

BOOL TimeMgt_RemoveCyclicEventByTag(char *tag_u8p)
{
	BOOL ret_b;

	// TODO : factorise with first / next (Lionel method) with static pointer / rover
	if(list_T.head != NULL) // If list is not empty
	{
		tTimeMgt_Element *element = list_T.head;

		if (strncmp(element->tag_u8a, tag_u8p, 8) != 0U) // Specific test for head
		{
			list_T.head = list_T.head->next_p;
			free(element);
			ret_b = TRUE;
		}
		else  // If more than one element in list
		{
			do
			{
				if (element->next_p != NULL)
				{
					if (strncmp(element->next_p->tag_u8a, tag_u8p, 8) != 0U) // next object is the object to be removed
					{
						tTimeMgt_Element *element_next = element->next_p; // Local copy of the pointer to the next object

						element->next_p = element_next->next_p; // Change pointer

						free(element_next); // Free object that must be removed
						ret_b = TRUE;
					}
				}

				element = element->next_p;
			}
			while (element != NULL);
		}
	}

	return (ret_b);
}

void TimeMgt_EnableCyclicEvent(uint8_t channel_u8)
{

}

BOOL TimeMgt_EnableCyclicEventById(uint32_t id_u32)
{
	BOOL ret_b = FALSE;

	tTimeMgt_Element *match;
	tTimeMgt_Element *prev;

	if (TimeMgt_SearchEventById(id_u32, &prev, &match) == TRUE) // Search for the object
	{
		((tTimeMgt_Channel*)(match->item_p))->enable_b = TRUE;
	}

	return (ret_b);
}

BOOL TimeMgt_EnableCyclicEventByTag(char *tag_u8p)
{
	BOOL ret_b = FALSE;

	if(list_T.head != NULL) // If list is not empty
	{
		tTimeMgt_Element *element = list_T.head;

		if (strncmp(element->tag_u8a, tag_u8p, 8) != 0U) // Specific test for head
		{
			((tTimeMgt_Channel*)(list_T.head->item_p))->enable_b = TRUE;
			ret_b = TRUE;
		}
		else  // If more than one element in list
		{
			do
			{
				if (element->next_p != NULL)
				{
					if (strncmp(element->next_p->tag_u8a, tag_u8p, 8) != 0U) // next object is the object to be removed
					{
						((tTimeMgt_Channel*)(element->next_p->item_p))->enable_b = TRUE;
						ret_b = TRUE;
					}
				}

				element = element->next_p;
			}
			while (element != NULL);
		}
	}

	return (ret_b);
}

void TimeMgt_DisableCyclicEvent(uint8_t channel_u8)
{

}

BOOL TimeMgt_DisableCyclicEventById(uint32_t id_u32)
{
	BOOL ret_b = FALSE;

	tTimeMgt_Element *match;
	tTimeMgt_Element *prev;

	if (TimeMgt_SearchEventById(id_u32, &prev, &match) == TRUE) // Search for the object
	{
		((tTimeMgt_Channel*)(match->item_p))->enable_b = FALSE;
	}

	return (ret_b);
}

BOOL TimeMgt_DisableCyclicEventByTag(char *tag_u8p)
{
	BOOL ret_b = FALSE;

	if(list_T.head != NULL) // If list is not empty
	{
		tTimeMgt_Element *element = list_T.head;

		if (strncmp(element->tag_u8a, tag_u8p, 8) != 0U) // Specific test for head
		{
			((tTimeMgt_Channel*)(list_T.head->item_p))->enable_b = TRUE;
			ret_b = TRUE;
		}
		else  // If more than one element in list
		{
			do
			{
				if (element->next_p != NULL)
				{
					if (strncmp(element->next_p->tag_u8a, tag_u8p, 8) != 0U) // next object is the object to be removed
					{
						((tTimeMgt_Channel*)(element->next_p->item_p))->enable_b = TRUE;
						ret_b = TRUE;
					}
				}

				element = element->next_p;
			}
			while (element != NULL);
		}
	}

	return (ret_b);
}

void TimeMgt_PeriodicFunction(void)
{
	if (list_T.head != NULL)
	{
		tTimeMgt_Element *element = list_T.head;

		while(element != NULL)
		{
			if (((tTimeMgt_Channel*)(element->item_p))->enable_b == TRUE)
			{
				if (((tTimeMgt_Channel*)(element->item_p))->counter_u32 < ((tTimeMgt_Channel*)(element->item_p))->timeout_u32)
				{
					((tTimeMgt_Channel*)(element->item_p))->counter_u32++;
				}
				else
				{
					((tTimeMgt_Channel*)(element->item_p))->counter_u32 = 0U;
					if (((tTimeMgt_Channel*)(element->item_p))->callback_p != NULL)
					{
						((tTimeMgt_Channel*)(element->item_p))->callback_p();
					}
				}
			}

			element = element->next_p;
		}
	}
}


